package ru.amse.baltijsky.javascheme.frames.image;

import ru.amse.baltijsky.javascheme.exporter.image.IImageExporter;
import ru.amse.baltijsky.javascheme.nodeshape.INodeShape;
import ru.amse.baltijsky.javascheme.util.Dir;
import ru.amse.baltijsky.javascheme.util.Size;
import ru.amse.baltijsky.javascheme.util.VisConst;

import javax.swing.*;
import java.awt.*;

/**
 * The panel where the image representing the schema tree is drawn.
 */
public class ImageViewPanel extends JPanel implements Scrollable {
    /**
     * The image exporter.
     */
    private final IImageExporter exporter;
    /**
     * The drawing point x-coordinate.
     */
    private int xDrawingCoord;
    /**
     * Debugging message output.
     */
    private boolean messageShown = false;
    /**
     * Image dimension in the graphics environment present at the moment of the last <code>paintComponent</code> invokation.
     */
    private Dimension imageSize = new Dimension(VisConst.PREFERRED_SCROLLER_WIDTH, VisConst.PREFERRED_SCROLLER_HEIGHT);

    /**
     * Creates a new schema tree image view panel.
     *
     * @param imageExporter image exporter
     */
    public ImageViewPanel(IImageExporter imageExporter) {
        exporter = imageExporter;
        setBackground(Color.WHITE);

        // temporary
//        final JComponent panel = this;
//        addMouseListener(new MouseAdapter() {
//            @Override
//            public void mouseClicked(MouseEvent e) {
//                JOptionPane.showMessageDialog(panel, "(" + e.getX() + ", " + e.getY() + ")");
//            }
//        });

    }

    public void start() {
        xDrawingCoord = 0;
        Graphics g = getGraphics();
        g.setFont(VisConst.FONT);
        Size imageSize = exporter.getImageSize(g);
        setPreferredSize(new Dimension(imageSize.x, imageSize.y));
        revalidate();
    }


    public void resizeToImage(Graphics g) {
        Size size = exporter.getImageSize(g);
        setSize(size.x, size.y);
    }

    public Dimension getSchemeSize(Graphics g) {
        return exporter.getImageSize(g).toDimension();
    }

    public Dimension getImageSize() {
        return imageSize;
    }

    public IImageExporter getExporter() {
        return exporter;
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);

        // setup graphics
        Graphics2D graphics2d = (Graphics2D) g;
        graphics2d.addRenderingHints(new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON));
        graphics2d.addRenderingHints(new RenderingHints(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB));

        g.setFont(VisConst.FONT);
        // set the drawing point
        Size szImage = exporter.getImageSize(g);
        imageSize = new Dimension(szImage.x + 1, szImage.y + 1);
        xDrawingCoord = (getSize().width - imageSize.width) / 2;
        setPreferredSize(imageSize);
        exporter.draw(g, xDrawingCoord, 0);
    }

    public Dimension getPreferredScrollableViewportSize() {
        return imageSize;
    }

    public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
        return 10;
    }

    public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
        return 10;
    }

    public boolean getScrollableTracksViewportWidth() {
        return false;
    }

    public boolean getScrollableTracksViewportHeight() {
        return false;
    }

    /**
     * Debugging feature  draws the specified shape only.
     *
     * @param shapeClass the shape to draw
     * @param caption    caption in the shape
     * @param g          graphics context
     */
    private void testShape(Class<? extends INodeShape> shapeClass, String caption, Graphics g) {
        try {
            INodeShape shape = shapeClass.getConstructor(String.class).newInstance(caption);
            shape.draw(shape.getShapeDims(g)[Dir.l()] + VisConst.XGAP, shape.getShapeDims(g)[Dir.u()] + VisConst.YGAP, g);
        } catch (Exception e) {
            if (!messageShown) {
                messageShown = true;
                JOptionPane.showMessageDialog(this, "cannot test shape " + shapeClass.toString() + ": " + e.getMessage());
            }
        }
    }
}
