package ru.amse.baltijsky.javascheme.io.importer.sablecc.java14;

import ru.amse.baltijsky.javascheme.io.importer.sablecc.java14.analysis.DepthFirstAdapter;
import ru.amse.baltijsky.javascheme.io.importer.sablecc.java14.node.*;

import java.util.ArrayList;
import java.util.List;

/**
 * Extracts method signature from the given method header (PMethodHeader).
 */
class SignatureExtractor extends DepthFirstAdapter {
    /**
     * List of parameters types extracted from method header.
     */
    private List<PType> params = new ArrayList<PType>();
    private String name;

    /**
     * Extracts method signature from the given method header.
     *
     * @param header the method header
     * @return string containing the signature in the form <code> <name>([[<type>,]*<type>]?) </code>
     */
    public String getMethodSignature(PMethodHeader header) {
        header.apply(this);
        return name + "(" + getArgsTypeList() + ")";
    }

    public String getArgsTypeList() {
        StringBuilder argsTypeList = new StringBuilder("");
        for (int i = 0; i < params.size(); ++i) {
            argsTypeList.append(params.get(i).toString().trim());
            if (i < params.size() - 1) {
                argsTypeList.append(", ");
            }
        }
        params.clear();
        return argsTypeList.toString();
    }

    @Override
    public void caseATypeMethodHeader(ATypeMethodHeader header) {
        header.getMethodDeclarator().apply(this);
    }

    @Override
    public void caseAVoidMethodHeader(AVoidMethodHeader header) {
        header.getMethodDeclarator().apply(this);
    }

    @Override
    public void caseAMethodDeclaratorMethodDeclarator(AMethodDeclaratorMethodDeclarator decl) {
        decl.getMethodDeclarator().apply(this);
    }

    @Override
    public void caseAIdentifierMethodDeclarator(AIdentifierMethodDeclarator decl) {
        name = decl.getIdentifier() != null ? decl.getIdentifier().toString() : "";
        if (decl.getFormalParameterList() != null) {
            decl.getFormalParameterList().apply(this);
        }
    }

    @Override
    public void caseAFormalParameterFormalParameterList(AFormalParameterFormalParameterList list) {
        params.add(((AFormalParameter) list.getFormalParameter()).getType());
    }

    @Override
    public void caseAFormalParameterListFormalParameterList(AFormalParameterListFormalParameterList list) {
        list.getFormalParameterList().apply(this);
        params.add(((AFormalParameter) list.getFormalParameter()).getType());
    }
}
