package ru.amse.baltijsky.javascheme.importer.sablecc.java15;

import ru.amse.baltijsky.javascheme.importer.sablecc.java15.analysis.DepthFirstAdapter;
import ru.amse.baltijsky.javascheme.importer.sablecc.java15.node.AFormalParameter;
import ru.amse.baltijsky.javascheme.importer.sablecc.java15.node.AOneFormalParameterList;
import ru.amse.baltijsky.javascheme.importer.sablecc.java15.node.ASomeFormalParameterList;
import ru.amse.baltijsky.javascheme.importer.sablecc.java15.node.PFormalParameters;

import java.util.ArrayList;

/**
 * Returns a pretty-printed string for some grammar constructions.
 */
class PrettyPrinter extends DepthFirstAdapter {
    private ArrayList<String> formalParameters = new ArrayList<String>();
    private ArrayList<String> methodSignature = new ArrayList<String>();

    public String retrieveParameters(PFormalParameters pFormalParameters) {
        formalParameters.clear();
        pFormalParameters.apply(this);
        StringBuilder params = new StringBuilder();
        for (int i = formalParameters.size() - 1; i >= 0; --i) {
            params.append(formalParameters.get(i).trim()).append(i > 0 ? ", " : "");
        }
        return params.toString();
    }

    /**
     * Retrieves method signature  the features identifying the method: its name and list of formal parameters.
     *
     * @param methodName        the name of the method
     * @param pFormalParameters an AST node for formal parameters
     * @return prettyprinted method signature
     */
    public String retrieveSignature(String methodName, PFormalParameters pFormalParameters) {
        methodSignature.clear();
        pFormalParameters.apply(this);
        StringBuilder params = new StringBuilder();
        for (int i = methodSignature.size() - 1; i >= 0; --i) {
            params.append(methodSignature.get(i).trim()).append(i > 0 ? ", " : "");
        }
        return methodName + "(" + params.toString() + ")";
    }

    public void inAOneFormalParameterList(AOneFormalParameterList node) {
        formalParameters.add(node.toString());
        methodSignature.add(((AFormalParameter) node.getFormalParameter()).getType().toString());
    }

    public void inASomeFormalParameterList(ASomeFormalParameterList node) {
        formalParameters.add(node.getFormalParameter().toString());
        methodSignature.add(((AFormalParameter) node.getFormalParameter()).getType().toString());
    }

}
