package ru.amse.baltijsky.javascheme.nodeshape;

import ru.amse.baltijsky.javascheme.util.Size;
import ru.amse.baltijsky.javascheme.util.VisConst;

import java.awt.*;

/**
 * Represents a hexagonal shape for the While node housing the code frame.
 * The hexagon consists of a central rectangular part and two isosceles triangular ears.
 */
public class WhileNodeShape extends AbstractNodeShape {
    /**
     * The main angle of the ear triangles (they are isosceles).
     */
    private static final double ALPHA = Math.PI / 8;
    /**
     * The offset for the code frame inside the central rectangular part.
     */
    private static final Size OFFSET = new Size(5, 5);

    /**
     * Creates a new shape for While node that contains the specified code.
     *
     * @param code the node code
     */
    public WhileNodeShape(String code) {
        super(code);
    }

    public void draw(int x, int y, Graphics g) {
        Size size = getBoundRectSize(g);
        int halfX = size.x / 2;
        int halfY = size.y / 2;
        int ear = (int) (halfY * Math.tan(ALPHA));
        Color oldColor = g.getColor();

        // draw a hexagon
        g.setColor(VisConst.SCHEME_COL);
        g.drawLine(x - halfX, y, x - halfX + ear, y - halfY);
        g.drawLine(x - halfX + ear, y - halfY, x + halfX - ear, y - halfY);
        g.drawLine(x + halfX - ear, y - halfY, x + halfX, y);
        g.drawLine(x + halfX, y, x + halfX - ear, y + halfY);
        g.drawLine(x + halfX - ear, y + halfY, x - halfX + ear, y + halfY);
        g.drawLine(x - halfX + ear, y + halfY, x - halfX, y);
        g.drawLine(x - halfX + ear, y - halfY, x - halfX + ear, y + halfY);
        g.drawLine(x + halfX - ear, y - halfY, x + halfX - ear, y + halfY);

        // draw label
        g.setColor(VisConst.TEXT_COL);
        g.drawString(getCode(), x - halfX + ear + OFFSET.x, y + OFFSET.y);

        g.setColor(oldColor);
    }

    public Size getBoundRectSize(Graphics g) {
        Size codeSize = getCodeSize(g);
        int basicSizeX = OFFSET.x + codeSize.x + OFFSET.x;
        int basicSizeY = OFFSET.y + codeSize.y + OFFSET.y;
        int sizeX = (int) (basicSizeX + basicSizeY * Math.tan(ALPHA));

        return new Size(sizeX, basicSizeY);
    }
}
