package ru.amse.baltijsky.javascheme.tree;

import ru.amse.baltijsky.javascheme.nodeshape.INodeShape;
import ru.amse.baltijsky.javascheme.nodeshape.SwitchNodeShape;

/**
 * Represents a <code>switch</code>-statement (conditional statement).
 * May have several previous nodes (join) and several (one or more) next nodes (fork).
 */
public class SwitchNode extends BranchNode {
    /**
     * Default constructor. Creates a node without previous or parent (nesting) node.
     */
    public SwitchNode() {
        super();
    }

    public SwitchNode(SchemaNode parentNode, SchemaNode prevNode) {
        super(parentNode, prevNode);
    }

    /**
     * Constructs a new node without previous or nesting node and with information about the corresponding code.
     *
     * @param s code associated with the node
     */
    public SwitchNode(String[] s) {
        super(s);
    }

    /**
     * Constructs a new node with full information about the nesting and previous nodes and the corresponding code.
     *
     * @param parentNode link to the parent (nesting) node
     * @param prevNode   link to the node that is previous in the sense of program flow
     * @param s          code associated with the node
     */
    public SwitchNode(SchemaNode parentNode, SchemaNode prevNode, String[] s) {
        super(parentNode, prevNode, s);
    }

    public INodeShape getShape() {
        return new SwitchNodeShape("switch");
    }

    public NodeType getNodeType() {
        return NodeType.SWITCH;
    }
}
